@@ -10,9 +10,11 @@ has attr => (
   handles => {
     'my_abs' => 'abs',
     'my_add' => 'add',
+    'my_ceil' => 'ceil',
     'my_cmp' => 'cmp',
     'my_div' => 'div',
     'my_eq' => 'eq',
+    'my_floor' => 'floor',
     'my_ge' => 'ge',
     'my_get' => 'get',
     'my_gt' => 'gt',
